/*
 * icon_view.c or icon_view.h
 * Desklet that displays icons on desktop
 *
 * Copyright (C) 2012 - Akash Rawal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COMMON_H_INSIDE
//icon_view.c
#include "common.h"
#else
//icon_view.h

typedef struct
{
	GObject parent;

	GVolumeMonitor *monitor;
	Settings *settings;
	gint changed;

	GtkWidget *desklet;
	GtkWidget *box;
} IconView;

typedef struct
{
	GObjectClass parent;

} IconViewClass;

//FILE_HEADER_SUBST:gobject_macro_gen ICON_VIEW IconView icon_view

#endif

//FILE_HEADER_END

#ifndef COMMON_H_INSIDE
G_DEFINE_TYPE(IconView, icon_view, G_TYPE_OBJECT);
#else
GType icon_view_get_type();
#endif

static void icon_view_class_init(IconViewClass *klass)
{

}

void icon_view_create_desklet(IconView *self)
{
	if (self->desklet) gtk_widget_destroy(self->desklet);
	self->desklet = desklet_new();
	gtk_window_set_resizable(GTK_WINDOW(self->desklet), FALSE);
	/*
	gchar *geometry = g_strdup_printf("%c%d%c%d",
	                                  (self->settings->x < 0 ? '-' : '+'),
	                                  ABS(self->settings->x),
	                                  (self->settings->y < 0 ? '-' : '+'),
	                                  ABS(self->settings->y));
	gtk_window_parse_geometry(GTK_WINDOW(self->desklet), geometry);
	g_free(geometry);
	*/
	
	if (self->settings->vertical)
		self->box = gtk_vbox_new(FALSE, 0);
	else
		self->box = gtk_hbox_new(FALSE, 0);

	if (self->settings->show_frame)
	{
		GtkWidget *frame = gtk_frame_new(NULL);
		gtk_container_add(GTK_CONTAINER(frame), self->box);
		gtk_container_add(GTK_CONTAINER(self->desklet), frame);
	}
	else
	{
		gtk_container_add(GTK_CONTAINER(self->desklet), self->box);
	}

	GList *list = g_volume_monitor_get_volumes(self->monitor);
	GList *iter;
	for (iter = g_list_first(list); iter; iter = iter->next)
	{
		GtkWidget *icon = g_object_new(TYPE_ICON, NULL);
		icon_set_volume(ICON(icon), G_VOLUME(iter->data));
		gtk_box_pack_start(GTK_BOX(self->box), icon, FALSE, FALSE, 0);
	}

	g_list_free(list);

	gtk_widget_show_all(self->desklet);

	//Placement
	desklet_set_position(DESKLET(self->desklet), self->settings->x, self->settings->y);
}

static void icon_view_init(IconView *self)
{
	self->settings = SETTINGS(g_object_new(TYPE_SETTINGS, NULL));
	self->monitor = g_volume_monitor_get();

	icon_view_create_desklet(self);
	
	g_signal_connect_swapped(self->monitor, "volume-added",
	                         G_CALLBACK(icon_view_add_icon), self);
	g_signal_connect_swapped(self->settings, "changed",
	                         G_CALLBACK(icon_view_changed), self);
	
}

void icon_view_add_icon(IconView *self, GVolume *volume)
{
	GtkWidget *icon = g_object_new(TYPE_ICON, NULL);
	icon_set_volume(ICON(icon), volume);
	gtk_box_pack_start(GTK_BOX(self->box), icon, FALSE, FALSE, 0);
	gtk_widget_show_all(icon);
}

void icon_view_changed(IconView *self, gint hint)
{
	if (! self->changed)
		g_timeout_add(300, (GSourceFunc) icon_view_changed_timeout, self);

	self->changed |= hint;
}

gboolean icon_view_changed_timeout(IconView *self)
{
	if (self->changed & SETTINGS_THEME)
		gtk_widget_queue_draw(self->desklet);
	else if (self->changed & SETTINGS_WINDOW)
		desklet_set_position(DESKLET(self->desklet),
		                     self->settings->x, self->settings->y);
	else if (self->changed & SETTINGS_OTHER)
		icon_view_create_desklet(self);
	
	self->changed = 0;
	return FALSE;
}

